@@ -5,6 +5,7 @@ import os
 import re
 import subprocess
 from collections import defaultdict, OrderedDict
+from datetime import datetime
 from . import config
 from . import dfnpanels
 from .refs import utils as refUtils
@@ -150,6 +151,32 @@ def addStatusSection(doc):
     fillWith('status', parseHTML(html), doc=doc)
 
 
+def addExpiryNotice(doc):
+    if doc.md.expires is None:
+        return
+    if doc.md.date >= doc.md.expires or datetime.utcnow().date() >= doc.md.expires:
+        boilerplate = "warning-expired"
+    else:
+        boilerplate = "warning-expires"
+        doc.extraScripts['script-expires'] = expiryScript
+    html = config.retrieveBoilerplateFile(doc, boilerplate)
+    html = doc.fixText(html)
+    fillWith('warning', parseHTML(html), doc=doc)
+    addClass(doc.body, boilerplate)
+
+expiryScript = '''
+const warning = document.querySelector('#expiry-notice');
+const expiresOn = warning.dataset.expires;
+const today = new Date().toISOString();
+if(expires < today) {
+    warning.setAttribute("open", "");
+    for(const swap of warning.querySelectorAll("[data-after-expiry]")) {
+        swap.textContent = swap.dataset.afterExpiry;
+    }
+}
+'''
+
+
 def addObsoletionNotice(doc):
     if doc.md.warning:
         html = config.retrieveBoilerplateFile(doc, doc.md.warning[0])
