@@ -8,8 +8,9 @@ import os
 import re
 import subprocess
 from collections import defaultdict, OrderedDict
-from datetime import datetime
+from datetime import datetime, timedelta, date
 from functools import partial
+from isodate import parse_duration, Duration
 
 from . import attr
 from . import config
@@ -67,6 +68,7 @@ class MetadataManager:
         self.displayShortname = None
         self.editors = []
         self.editorTerm = {"singular": "Editor", "plural": "Editors"}
+        self.expires = None
         self.favicon = None
         self.forceCrossorigin = False
         self.group = None
@@ -171,6 +173,8 @@ class MetadataManager:
             self.issues.append(("GitHub", self.repository.formatIssueUrl()))
         self.status = config.canonicalizeStatus(self.rawStatus, self.group)
 
+        self.expires = canonicalizeExpiryDate(self.date, self.expires)
+
         # for TR document, defaults to TR
         # fallbacks to ED
         if (self.canonicalURL == None or self.canonicalURL == "TR") and self.TR:
@@ -280,6 +284,13 @@ class MetadataManager:
         macros["isodate"] = self.date.strftime("%Y-%m-%d")
         macros["date-my"] = self.date.strftime("%b %Y")
         macros["date-mmy"] = self.date.strftime("%B %Y")
+        if isinstance(self.expires, date):
+            macros["expires"] = self.expires.strftime("{0} %B %Y".format(self.expires.day))
+            macros["expires-dmmy"] = self.expires.strftime("{0} %B %Y".format(self.expires.day)) #same as plain 'expires'
+            macros["cexpires"] = self.expires.strftime("%Y%m%d")
+            macros["isoexpires"] = self.expires.strftime("%Y-%m-%d")
+            macros["expires-my"] = self.expires.strftime("%b %Y")
+            macros["expires-mmy"] = self.expires.strftime("%B %Y")
         if self.deadline:
             macros["deadline"] = self.deadline.strftime("{0} %B %Y".format(self.deadline.day))
             macros["isodeadline"] = self.deadline.strftime("%Y-%m-%d")
@@ -334,6 +345,34 @@ def parseDate(key, val, lineNum):
         return None
 
 
+def parseDateOrDuration(key, val, lineNum):
+    if val == "now":
+        return datetime.utcnow().date()
+    if val == "never" or boolish(val) is False:
+        return None
+    try:
+        if val.startswith("P"):
+            return parse_duration(val)
+        return datetime.strptime(val, "%Y-%m-%d").date()
+    except:
+        die("The {0} field must be an ISO 8601 duration, a date in the format YYYY-MM-DD, now, never, false, no, n, or off. Got '{1}' instead.", key, val, lineNum=lineNum)
+        return None
+
+def canonicalizeExpiryDate(base, expires):
+    if expires is None:
+        return None
+    if isinstance(expires, timedelta):
+        return base + expires
+    if isinstance(expires, Duration):
+        return base + expires
+    if isinstance(expires, datetime):
+        return expires.date()
+    if isinstance(expires, date):
+        return expires
+    die("Unexpected expiry type: canonicalizeExpiryDate({0}, {1})", base, expires)
+    return None
+
+
 def parseLevel(key, val, lineNum):
     val = val.lower().strip()
     if val == "none":
@@ -992,6 +1031,7 @@ knownKeys = {
     "ED": Metadata("ED", "ED", joinValue, parseLiteral),
     "Editor": Metadata("Editor", "editors", joinList, parseEditor),
     "Editor Term": Metadata("Editor Term", "editorTerm", joinValue, parseEditorTerm),
+    "Expires": Metadata("Expires", "expires", joinValue, parseDateOrDuration),
     "Favicon": Metadata("Favicon", "favicon", joinValue, parseLiteral),
     "Force Crossorigin": Metadata("Force Crossorigin", "forceCrossorigin", joinValue, parseBoolean),
     "Former Editor": Metadata("Former Editor", "previousEditors", joinList, parseEditor),
